@@ -1,18 +1,18 @@
 /* Auxiliary functions for the creation of subprocesses.  Native Windows API.
-   Copyright (C) 2001, 2003-2020 Free Software Foundation, Inc.
+   Copyright (C) 2001, 2003-2024 Free Software Foundation, Inc.
    Written by Bruno Haible <bruno@clisp.org>, 2003.
 
-   This program is free software: you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3 of the License, or
-   (at your option) any later version.
+   This file is free software: you can redistribute it and/or modify
+   it under the terms of the GNU Lesser General Public License as
+   published by the Free Software Foundation; either version 2.1 of the
+   License, or (at your option) any later version.
 
-   This program is distributed in the hope that it will be useful,
+   This file is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU Lesser General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
+   You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 #include <config.h>
@@ -24,7 +24,6 @@
 #define WIN32_LEAN_AND_MEAN
 #include <windows.h>
 
-#include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -39,6 +38,7 @@
 #include <process.h>
 
 #include "findprog.h"
+#include "windows-path.h"
 
 /* Don't assume that UNICODE is not defined.  */
 #undef STARTUPINFO
@@ -245,7 +245,7 @@ compose_command (const char * const *argv)
 }
 
 char *
-compose_envblock (const char * const *envp)
+compose_envblock (const char * const *envp, const char *new_PATH)
 {
   /* This is a bit hairy, because we don't have a lock that would prevent other
      threads from making modifications in ENVP.  So, just make sure we don't
@@ -258,8 +258,11 @@ compose_envblock (const char * const *envp)
     size_t total_size = 0;
     const char * const *ep;
     const char *p;
+    if (new_PATH != NULL)
+      total_size += strlen (new_PATH) + 1;
     for (ep = envp; (p = *ep) != NULL; ep++)
-      total_size += strlen (p) + 1;
+      if (!(new_PATH != NULL && strncmp (p, "PATH=", 5) == 0))
+        total_size += strlen (p) + 1;
     size_t envblock_size = total_size;
 
     /* Allocate the block of memory.  */
@@ -270,34 +273,42 @@ compose_envblock (const char * const *envp)
         return NULL;
       }
     size_t envblock_used = 0;
-    for (ep = envp; (p = *ep) != NULL; ep++)
+    if (new_PATH != NULL)
       {
-        size_t size = strlen (p) + 1;
-        if (envblock_used + size > envblock_size)
-          {
-            /* Other threads did modifications.  Need more memory.  */
-            envblock_size += envblock_size / 2;
-            if (envblock_used + size > envblock_size)
-              envblock_size = envblock_used + size;
-
-            char *new_envblock = (char *) realloc (envblock, envblock_size + 1);
-            if (new_envblock == NULL)
-              {
-                free (envblock);
-                errno = ENOMEM;
-                return NULL;
-              }
-            envblock = new_envblock;
-          }
-        memcpy (envblock + envblock_used, p, size);
+        size_t size = strlen (new_PATH) + 1;
+        memcpy (envblock + envblock_used, new_PATH, size);
         envblock_used += size;
-        if (envblock[envblock_used - 1] != '\0')
-          {
-            /* Other threads did modifications.  Restart.  */
-            free (envblock);
-            goto retry;
-          }
       }
+    for (ep = envp; (p = *ep) != NULL; ep++)
+      if (!(new_PATH != NULL && strncmp (p, "PATH=", 5) == 0))
+        {
+          size_t size = strlen (p) + 1;
+          if (envblock_used + size > envblock_size)
+            {
+              /* Other threads did modifications.  Need more memory.  */
+              envblock_size += envblock_size / 2;
+              if (envblock_used + size > envblock_size)
+                envblock_size = envblock_used + size;
+
+              char *new_envblock =
+                (char *) realloc (envblock, envblock_size + 1);
+              if (new_envblock == NULL)
+                {
+                  free (envblock);
+                  errno = ENOMEM;
+                  return NULL;
+                }
+              envblock = new_envblock;
+            }
+          memcpy (envblock + envblock_used, p, size);
+          envblock_used += size;
+          if (envblock[envblock_used - 1] != '\0')
+            {
+              /* Other threads did modifications.  Restart.  */
+              free (envblock);
+              goto retry;
+            }
+        }
     envblock[envblock_used] = '\0';
     return envblock;
   }
@@ -324,14 +335,21 @@ init_inheritable_handles (struct inheritable_handles *inh_handles,
         HANDLE handle = (HANDLE) _get_osfhandle (fd);
         if (handle != INVALID_HANDLE_VALUE)
           {
-            DWORD hflags;
-            /* GetHandleInformation
-               <https://docs.microsoft.com/en-us/windows/win32/api/handleapi/nf-handleapi-gethandleinformation>  */
-            if (GetHandleInformation (handle, &hflags))
+            if (duplicate)
+              /* We will add fd to the array, regardless of whether it is
+                 inheritable or not.  */
+              break;
+            else
               {
-                if ((hflags & HANDLE_FLAG_INHERIT) != 0)
-                  /* fd denotes an inheritable descriptor.  */
-                  break;
+                DWORD hflags;
+                /* GetHandleInformation
+                   <https://docs.microsoft.com/en-us/windows/win32/api/handleapi/nf-handleapi-gethandleinformation>  */
+                if (GetHandleInformation (handle, &hflags))
+                  {
+                    if ((hflags & HANDLE_FLAG_INHERIT) != 0)
+                      /* fd denotes an inheritable descriptor.  */
+                      break;
+                  }
               }
           }
       }
@@ -339,31 +357,23 @@ init_inheritable_handles (struct inheritable_handles *inh_handles,
   }
   /* Note: handles_count >= 3.  */
 
-  /* Allocate the arrays.  */
+  /* Allocate the array.  */
   size_t handles_allocated = handles_count;
-  HANDLE *handles_array =
-    (HANDLE *) malloc (handles_allocated * sizeof (HANDLE));
-  if (handles_array == NULL)
+  struct IHANDLE *ih =
+    (struct IHANDLE *) malloc (handles_allocated * sizeof (struct IHANDLE));
+  if (ih == NULL)
     {
       errno = ENOMEM;
       return -1;
     }
-  unsigned char *flags_array =
-    (unsigned char *) malloc (handles_allocated * sizeof (unsigned char));
-  if (flags_array == NULL)
-    {
-      free (handles_array);
-      errno = ENOMEM;
-      return -1;
-    }
 
-  /* Fill in the two arrays.  */
+  /* Fill in the array.  */
   {
     HANDLE curr_process = (duplicate ? GetCurrentProcess () : INVALID_HANDLE_VALUE);
     unsigned int fd;
     for (fd = 0; fd < handles_count; fd++)
       {
-        handles_array[fd] = INVALID_HANDLE_VALUE;
+        ih[fd].handle = INVALID_HANDLE_VALUE;
         /* _get_osfhandle
            <https://docs.microsoft.com/en-us/cpp/c-runtime-library/reference/get-osfhandle>  */
         HANDLE handle = (HANDLE) _get_osfhandle (fd);
@@ -374,29 +384,42 @@ init_inheritable_handles (struct inheritable_handles *inh_handles,
                <https://docs.microsoft.com/en-us/windows/win32/api/handleapi/nf-handleapi-gethandleinformation>  */
             if (GetHandleInformation (handle, &hflags))
               {
-                if ((hflags & HANDLE_FLAG_INHERIT) != 0)
+                if (duplicate)
                   {
-                    /* fd denotes an inheritable descriptor.  */
-                    if (duplicate)
+                    /* Add fd to the array, regardless of whether it is
+                       inheritable or not.  */
+                    if ((hflags & HANDLE_FLAG_INHERIT) != 0)
+                      {
+                        /* Instead of duplicating it, just mark it as shared.  */
+                        ih[fd].handle = handle;
+                        ih[fd].flags = KEEP_OPEN_IN_PARENT | KEEP_OPEN_IN_CHILD;
+                      }
+                    else
                       {
                         if (!DuplicateHandle (curr_process, handle,
-                                              curr_process, &handles_array[fd],
+                                              curr_process, &ih[fd].handle,
                                               0, TRUE, DUPLICATE_SAME_ACCESS))
                           {
                             unsigned int i;
                             for (i = 0; i < fd; i++)
-                              if (handles_array[i] != INVALID_HANDLE_VALUE)
-                                CloseHandle (handles_array[i]);
-                            free (flags_array);
-                            free (handles_array);
+                              if (ih[i].handle != INVALID_HANDLE_VALUE
+                                  && !(ih[i].flags & KEEP_OPEN_IN_PARENT))
+                                CloseHandle (ih[i].handle);
+                            free (ih);
                             errno = EBADF; /* arbitrary */
                             return -1;
                           }
+                        ih[fd].flags = 0;
+                      }
+                  }
+                else
+                  {
+                    if ((hflags & HANDLE_FLAG_INHERIT) != 0)
+                      {
+                        /* fd denotes an inheritable descriptor.  */
+                        ih[fd].handle = handle;
+                        ih[fd].flags = KEEP_OPEN_IN_CHILD;
                       }
-                    else
-                      handles_array[fd] = handle;
-
-                    flags_array[fd] = 0;
                   }
               }
           }
@@ -406,8 +429,7 @@ init_inheritable_handles (struct inheritable_handles *inh_handles,
   /* Return the result.  */
   inh_handles->count = handles_count;
   inh_handles->allocated = handles_allocated;
-  inh_handles->handles = handles_array;
-  inh_handles->flags = flags_array;
+  inh_handles->ih = ih;
   return 0;
 }
 
@@ -418,9 +440,9 @@ compose_handles_block (const struct inheritable_handles *inh_handles,
   /* STARTUPINFO
      <https://docs.microsoft.com/en-us/windows/win32/api/processthreadsapi/ns-processthreadsapi-startupinfoa>  */
   sinfo->dwFlags = STARTF_USESTDHANDLES;
-  sinfo->hStdInput  = inh_handles->handles[0];
-  sinfo->hStdOutput = inh_handles->handles[1];
-  sinfo->hStdError  = inh_handles->handles[2];
+  sinfo->hStdInput  = inh_handles->ih[0].handle;
+  sinfo->hStdOutput = inh_handles->ih[1].handle;
+  sinfo->hStdError  = inh_handles->ih[2].handle;
 
   /* On newer versions of Windows, more file descriptors / handles than the
      first three can be passed.
@@ -471,11 +493,11 @@ compose_handles_block (const struct inheritable_handles *inh_handles,
         handles_aligned[fd] = INVALID_HANDLE_VALUE;
         flags[fd] = 0;
 
-        HANDLE handle = inh_handles->handles[fd];
+        HANDLE handle = inh_handles->ih[fd].handle;
         if (handle != INVALID_HANDLE_VALUE
             /* The first three are possibly already passed above.
                But they need to passed here as well, if they have some flags.  */
-            && (fd >= 3 || inh_handles->flags[fd] != 0))
+            && (fd >= 3 || (unsigned char) inh_handles->ih[fd].flags != 0))
           {
             DWORD hflags;
             /* GetHandleInformation
@@ -490,7 +512,7 @@ compose_handles_block (const struct inheritable_handles *inh_handles,
                        flags[fd] = 1.  But on ReactOS or Wine, adding the bit
                        that indicates the handle type may be necessary.  So,
                        just do it everywhere.  */
-                    flags[fd] = 1 | inh_handles->flags[fd];
+                    flags[fd] = 1 | (unsigned char) inh_handles->ih[fd].flags;
                     switch (GetFileType (handle))
                       {
                       case FILE_TYPE_CHAR:
@@ -522,8 +544,7 @@ compose_handles_block (const struct inheritable_handles *inh_handles,
 void
 free_inheritable_handles (struct inheritable_handles *inh_handles)
 {
-  free (inh_handles->flags);
-  free (inh_handles->handles);
+  free (inh_handles->ih);
 }
 
 int
@@ -539,30 +560,24 @@ convert_CreateProcess_error (DWORD error)
     case ERROR_INVALID_NAME:
     case ERROR_DIRECTORY:
       return ENOENT;
-      break;
 
     case ERROR_ACCESS_DENIED:
     case ERROR_SHARING_VIOLATION:
       return EACCES;
-      break;
 
     case ERROR_OUTOFMEMORY:
       return ENOMEM;
-      break;
 
     case ERROR_BUFFER_OVERFLOW:
     case ERROR_FILENAME_EXCED_RANGE:
       return ENAMETOOLONG;
-      break;
 
     case ERROR_BAD_FORMAT:
     case ERROR_BAD_EXE_FORMAT:
       return ENOEXEC;
-      break;
 
     default:
       return EINVAL;
-      break;
     }
 }
 
@@ -570,6 +585,7 @@ intptr_t
 spawnpvech (int mode,
             const char *progname, const char * const *argv,
             const char * const *envp,
+            const char * const *dll_dirs,
             const char *currdir,
             HANDLE stdin_handle, HANDLE stdout_handle, HANDLE stderr_handle)
 {
@@ -597,11 +613,23 @@ spawnpvech (int mode,
 
   /* Copy *ENVP into a contiguous block of memory.  */
   char *envblock;
-  if (envp == NULL)
+  if (envp == NULL && !(dll_dirs != NULL && dll_dirs[0] != NULL))
     envblock = NULL;
   else
     {
-      envblock = compose_envblock (envp);
+      if (envp == NULL)
+        /* Documentation:
+           <https://learn.microsoft.com/en-us/cpp/c-runtime-library/environ-wenviron>  */
+        envp = (const char **) _environ;
+      char *new_PATH = NULL;
+      if (dll_dirs != NULL && dll_dirs[0] != NULL)
+        {
+          new_PATH = extended_PATH (dll_dirs);
+          if (new_PATH == NULL)
+            goto out_of_memory_2;
+        }
+      envblock = compose_envblock (envp, new_PATH);
+      free (new_PATH);
       if (envblock == NULL)
         goto out_of_memory_2;
     }
@@ -619,9 +647,12 @@ spawnpvech (int mode,
       errno = saved_errno;
       return -1;
     }
-  inh_handles.handles[0] = stdin_handle;  inh_handles.flags[0] = 0;
-  inh_handles.handles[1] = stdout_handle; inh_handles.flags[1] = 0;
-  inh_handles.handles[2] = stderr_handle; inh_handles.flags[2] = 0;
+  inh_handles.ih[0].handle = stdin_handle;
+  inh_handles.ih[0].flags = KEEP_OPEN_IN_CHILD;
+  inh_handles.ih[1].handle = stdout_handle;
+  inh_handles.ih[1].flags = KEEP_OPEN_IN_CHILD;
+  inh_handles.ih[2].handle = stderr_handle;
+  inh_handles.ih[2].flags = KEEP_OPEN_IN_CHILD;
 
   /* CreateProcess
      <https://docs.microsoft.com/en-us/windows/win32/api/processthreadsapi/nf-processthreadsapi-createprocessa>  */
