@@ -937,9 +937,9 @@ setval_internal_tilde (int comind, const char *com, const char *val)
   ret = setval_internal (comind, com, val);
 
   /* We make tilde expansion for cmd_file and cmd_directory */
-  if (((commands[comind].action == cmd_file) ||
+  if (ret && ((commands[comind].action == cmd_file) ||
        (commands[comind].action == cmd_directory))
-      && ret && (*val == '~' && ISSEP (val[1])))
+      && (*val == '~' && ISSEP (val[1])))
     {
       pstring = commands[comind].place;
       if (opt.homedir)
@@ -1938,7 +1938,7 @@ cleanup (void)
 
   log_close ();
 
-  if (output_stream && output_stream != stderr)
+  if (output_stream && output_stream != stderr && output_stream != stdout)
     {
       FILE *fp = output_stream;
       output_stream = NULL;
